- x
object of class "wrda", i.e. result of
wrda or cca0.
- ...
Other arguments passed to the function (currently ignored).
- choices
integer vector of which axes to obtain. Default: all wrda
axes.
- display
a character vector, one or more of c("all", "species",
"sites", "sp", "wa", "lc", "bp", "cor", "ic", "reg", "tval", "cn"). The
most items are as in scores.cca, except "cor"
and "ic", for inter-set and intra-set correlations, respectively,
and "tval" for the (over-optimistic) t-values of the regression
coefficients.
- scaling
numeric (1,2 or 3) or character "sites", "species" or
"symmetric". Default: "symmetric". Either site- (1) or species- (2) related
scores are scaled by eigenvalues, and the other set of scores have
unit weighted mean square or with 3 both are scaled symmetrically
to weighted mean squares equal to the square root of eigenvalues. Negative
values are treated as the corresponding positive ones by abs(scaling).
- which_cor
character vector environmental variables names in the data
frames for which inter-set correlations must calculated. Default: a
character ("in_model") for all predictors in the model, including collinear
variables and levels.
- normed
logical (default TRUE) giving standardized regression
coefficients and biplot scores. When FALSE, (regular)
regression coefficients and (unstandardized) biplot scores.
- tidy
Return scores that are compatible with ggplot2: all
variable score, the names by variable label. See
weights (in dc_CA are in variable weight. See
scores.cca.