| wrda | R Documentation |
wrda is formula-based implementation of weighted redundancy analysis.
wrda(
formula,
response = NULL,
data,
weights = rep(1/nrow(data), nrow(data)),
traceonly = FALSE,
cca_object = NULL,
object4QR = NULL
)
formula |
one or two-sided formula for the rows (samples) with row
predictors in |
response |
matrix or data frame of the abundance data (dimension
n x m). Rownames of |
data |
matrix or data frame of the row predictors, with rows
corresponding to those in |
weights |
row weights (a vector). If not specified unit weights are used. |
traceonly |
logical, default |
cca_object |
a vegan-type cca-object of transposed
|
object4QR |
a vegan-type cca-object with weighted QR's for
|
The algorithm is a modified version of published R-code for weighted redundancy analysis (ter Braak, 2022).
Compared to rda, wrda does not have residual
axes, i.e. no SVD or PCA of the residuals is performed.
All scores in the wrda object are in scaling "sites" (1):
the scaling with Focus on Case distances.
ter Braak C.J.F. and P. Ć milauer (2018). Canoco reference manual and user's guide: software for ordination (version 5.1x). Microcomputer Power, Ithaca, USA, 536 pp.
Oksanen, J., et al. (2022) vegan: Community Ecology Package. R package version 2.6-4. https://CRAN.R-project.org/package=vegan.
scores.wrda, anova.wrda,
print.wrda
data("dune_trait_env")
# rownames are carried forward in results
rownames(dune_trait_env$comm) <- dune_trait_env$comm$Sites
response <- dune_trait_env$comm[, -1] # must delete "Sites"
w <- rep(1, 20)
w[1:10] <- 8
w[17:20] <- 0.5
object <- wrda(formula = response ~ A1 + Moist + Mag + Use + Condition(Manure),
data = dune_trait_env$envir,
weights = w)
object # Proportions equal to those Canoco 5.15
mod_scores <- scores(object, display = "all")
scores(object, which_cor = c("A1", "X_lot"), display = "cor")
anova(object)
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.